import { Component, OnInit, ElementRef, Renderer2 } from '@angular/core';
import { DragulaService } from 'ng2-dragula/ng2-dragula';
import { AuthService } from '../services/auth.service';
import { ValidateService } from '../services/validate.service';
import { DialogService } from '../services/dialog.service';
import { MathService } from '../services/math.service';
import { ErrorService } from '../services/error.service';
import { AudioPlayerService } from '../services/audio-player.service';
import {saveAs as importedSaveAs} from "file-saver";

@Component({
  selector: 'app-spellbook',
  templateUrl: './spellbook.component.html',
  styleUrls: ['./spellbook.component.scss'],
  viewProviders: [DragulaService]
})
export class SpellbookComponent implements OnInit {

  pages:any[];
  newPage:string;
  currentPage:any;
  recommendation:any;
  aura:number;

  constructor(private dragula:DragulaService,
              private authService:AuthService,
              private validateService:ValidateService,
              private dialogService:DialogService,
              private ms:MathService,
              private errorService:ErrorService,
              private renderer:Renderer2,
              private audioService:AudioPlayerService) { }

  ngOnInit() {
    this.newPage = null;
    this.currentPage = null;
    this.recommendation = null;
    this.pages = null;
    this.aura = 0;

    this.currentPage = {
      name:"",
      isStatic:false,
      content: null
    };

    this.authService.authGet("spellbook").subscribe((p, err) => {
      this.errorService.serverError(err, p, () => {
        this.pages = p.list;
        this.currentPage = p.current;
        this.recommendation = p.recommendation;
        this.aura = p.aura;
      });
    });

    //register changes after drop
    this.dragula.drop.subscribe((value) => {
      let [bag, el, target, source, sibling] = value.splice(0);

      if(!el) {
        return;
      }

      let shift = 0;
      // drag pages
      if(bag == "bag-pages") {
        let dst = this.pages.length-1;
        if(sibling) {
          if(el.getAttribute('data-index') < sibling.getAttribute('data-index')) {
            shift = -1;
          }
          dst = Number(sibling.getAttribute('data-index')) + shift;
        }
        this.ms.arrayMoveElement(this.pages, Number(el.getAttribute('data-index')), dst);
        for(let i=0; i < this.pages.length; i++) {
          this.renderer.setAttribute(target.children[i], 'data-index', String(i));
        }

        this.authService.authPost("spellbook/movePage", "application/json", this.pages).subscribe((data, err) => {
          this.errorService.serverError(err, data, () => {});
        });
      }
    });
  }

  addNewPage() {
    if(this.newPage && this.validateService.validateListElem(this.newPage) &&
      !this.validateService.isIn(this.pages, this.newPage)) {

      this.authService.authPost("spellbook/newPage", "text/plain", this.newPage).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          let page = {
            name: data.name,
            id: data.id
          };
          this.pages.push(page);
          this.newPage = undefined;
        });
      });
    }
    else {
      this.newPage = "";
    }
  }

  changePage(p) {
    if(p.id != this.currentPage.id) {
      this.authService.authPost("spellbook/changePage", "application/json", p).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          this.currentPage = data.current;
          this.recommendation = data.recommendation;
        });
      });
    }
  }

  deletePage() {
    this.dialogService.confirm("Confirmation", "Voulez-vous vraiment supprimer cette page?").subscribe(res => {
      if(res) {
        for(let i=0; i < this.pages.length; i++) {
          if(this.pages[i].name == this.currentPage.name) {
            const id = this.pages[i].id;
            this.pages.splice(i, 1);

            this.authService.authPost("spellbook/deletePage", "application/json", {list: this.pages, id: id}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {
                this.currentPage = data.page;
                this.recommendation = data.recommendation;
              });
            });
            break;
          }
        }
      }
    });
  }

  renamePage() {
    this.dialogService.rename("Renommer", "Nouveau nom :").subscribe(res => {
      if(res && this.validateService.validateListElem(res)) {
        for(let p of this.pages) {
          if(p.name == this.currentPage.name) {
            this.currentPage.name = res;
            p.name = res;

            this.authService.authPost("spellbook/renamePage", "application/json", {list: this.pages, id: p.id, name: res}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {});
            });
            break;
          }
        }
      }
    });
  }

  deleteArcane(a) {
    this.dialogService.confirm("Confirmation", "Voulez-vous vraiment supprimer cet arcane?").subscribe(res => {
      if(res) {
        for(let i=0; i < this.currentPage.content.length; i++) {
          if(this.currentPage.content[i].id == a.id) {
            this.currentPage.content.splice(i, 1);
            this.authService.authPost("spellbook/deleteFromPage", "application/json", {pageId: this.currentPage.id, arcaneId: a.id, recommendation: false}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {});
            });
            break;
          }
        }
      }
    });
  }

  playSound(a) {
    this.audioService.changeSource(a);
  }

  addArcaneToPage(a) {
    //display only available pages
    this.authService.authPost("spellbook/pagePossibilities", "application/json", {arcaneId: a.id}).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        this.dialogService.addToPage("Ajouter l'arcane aux pages", data).subscribe(res => {
          if(res) {
            this.authService.authPost("spellbook/addToPage", "application/json", {pages: res, arcaneId: a.id, recommendation: false}).subscribe((dataAdd, err) => {
              this.errorService.serverError(err, dataAdd, () => {});
            });
          }
        });
      });
    });
  }

  canUnlock(a) {
    if(a.author == this.currentPage.author) {
      return false;
    }
    return true;
  }

  download(a) {
    let fileUrl = "sounds/" + a.fileName + ".WAV";
    this.authService.downloadFile(fileUrl).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        importedSaveAs(data, a.author + ".WAV");
      });
    });
  }

  unclock(a) {
    if(this.aura >= a.cost) {
      this.dialogService.confirm("Coût de décryptage", "Le décryptage de cet arcane vous coûtera : " + a.cost + " aura.").subscribe(res => {
        if(res) {
          this.authService.authPost("spellbook/decrypt/", "application/json", {arcane: a}).subscribe((data, err) => {
            this.errorService.serverError(err, data, () => {
              if(!data.success) {
                this.dialogService.info("décryptage impossible", data.msg);
              }
              else {
                this.aura = data.aura;
                this.dialogService.info("Réussite", "L'arcane décrypté est disponible dans l'espace de travail.");
              }
            });
          });
        }
      });
    }
    else {
      this.dialogService.info("décryptage impossible", "Vous ne possédez pas assez d'aura pour décrypter cet arcane.");
    }
  }


  //################## RECOMMENDATION #############################


  addRecommendation(a) {
    this.authService.authPost("spellbook/pagePossibilities", "application/json", {arcaneId: a.id}).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        this.dialogService.addToPage("Ajouter l'arcane aux pages", data).subscribe(res => {
          if(res) {
            this.authService.authPost("spellbook/addToPage", "application/json", {pages: res, arcaneId: a.id, pageId: this.currentPage.id, recommendation: true}).subscribe((dataAdd, err) => {
              this.errorService.serverError(err, dataAdd, () => {
                for(let r of res) {
                  if(r.id == this.currentPage.id) {
                    this.currentPage.content.push(a);
                    this.recommendation = dataAdd.recommendation;
                    this.aura = dataAdd.aura;
                    break;
                  }
                }
              });
            });
          }
        });
      });
    });
  }

  deleteRecommendation(a) {
    this.dialogService.confirm("Confirmation", "Voulez-vous vraiment supprimer cet recommandation?").subscribe(res => {
      if(res) {
        for(let i=0; i < this.recommendation.length; i++) {
          if(this.recommendation[i].id == a.id) {
            this.recommendation.splice(i, 1);
            this.authService.authPost("spellbook/deleteFromPage", "application/json", {pageId: this.currentPage.id, arcaneId: a.id, recommendation: true}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {
                this.recommendation = data.recommendation;
              });
            });
            break;
          }
        }
      }
    });
  }
}
