const chai = require('chai');
const chaiHttp = require('chai-http');
const app = require('../../server').app;
const udp = require('../../supercollider/udp');

chai.use(chaiHttp);

let formula = {
  size: [{type: "C", value: 8}],
  time: [{type: "T", value: 0.1}, {type: "T", value: 1.9}],
  amp: [{type: "env", value: [{type: "K", value: -1}, {type: "K/nil", value: "nil"}, {type: "K", value: 1}]}, {type: "mod", value: null}],
  freq: [{type: "param", value: [{type: "K/env", value: 0.2}, {type: "mod", value: null}]}],
  mass: [{type: "fonc1", value: [{type: "F1", value: "brt"}, {type: "param", value: [{type: "K/env", value: 1}, {type: "mod", value: null}]}]}],
  ton: [{type: "fonc2", value: null}],
  color: [{type: "param", value:null}, {type: "param", value:null}, {type: "param", value:null}]
}

module.exports = describe('Recommendation tests', () => {
    let authToken = null;
    let page = null;
    let arcaane = null;
    let recom = null;
    let list = null;

  describe('Getting recommendation with an empty database', () => {

    it('POST /users/register', (done) => {
      chai.request(app)
      .post('/users/register')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          email: 'user@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('POST /users/authenticate', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isDefined(res.body.token);
        chai.assert.equal(res.body.user.username, 'user');
        chai.assert.equal(res.body.user.email, 'user@mail.com');
        chai.assert.isUndefined(res.body.error);
        authToken = res.body.token;
        done();
      });
    });

    it('POST /spellbook/newPage', (done) => {
      chai.request(app)
      .post('/spellbook/newPage')
      .set('Authorization', authToken)
      .set('content-type', 'text/plain')
      .set('Cache-control', 'no-cache')
      .send("a")
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        page = res.body;
        done();
      });
    });

    it('POST /spellbook/changePage', (done) => {
      chai.request(app)
      .post('/spellbook/changePage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send(page)
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        chai.assert.equal(res.body.current.id, page.id);
        chai.assert.isUndefined(res.body.recommendation);
        done();
      });
    });
  });

  describe('Add first arcane to first page', () => {
    it('POST /generate', (done) => {
      udp.open();

      udp.on('ready', () => {
        chai.request(app)
        .post('/generate')
        .set('Authorization', authToken)
        .set('Content-Type', 'application/json')
        .set('Cache-control', 'no-cache')
        .send({
          content: formula
        })
        .end((err, res) => {
          chai.assert.isNull(err, err);
          chai.assert.isTrue(res.body.success);
          done();
        });
      });
    });

    it('GET /spellbook', (done) => {
      chai.request(app)
      .get('/spellbook')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        arcane = res.body.current.content[0];
        done();
      });
    });

    it('POST /spellbook/addToPage', (done) => {
      chai.request(app)
      .post('/spellbook/addToPage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        pages: [page],
        arcaneId: arcane.id,
        recommendation: false
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        done();
      });
    });

    it('POST /spellbook/changePage', (done) => {
      chai.request(app)
      .post('/spellbook/changePage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send(page)
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        chai.assert.equal(res.body.current.id, page.id);
        chai.assert.isUndefined(res.body.recommendation);
        page = res.body.current;
        done();
      });
    });
  });

  describe('New user should get one recommendation', () => {
    it('POST /users/register', (done) => {
      chai.request(app)
      .post('/users/register')
      .set('content-type', 'application/json')
      .send({
          username: 'user2',
          email: 'user2@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('POST /users/authenticate', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user2',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isDefined(res.body.token);
        chai.assert.equal(res.body.user.username, 'user2');
        chai.assert.equal(res.body.user.email, 'user2@mail.com');
        authToken = res.body.token;
        done();
      });
    });

    it('POST /spellbook/newPage', (done) => {
      chai.request(app)
      .post('/spellbook/newPage')
      .set('Authorization', authToken)
      .set('content-type', 'text/plain')
      .set('Cache-control', 'no-cache')
      .send("b")
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        page = res.body;
        done();
      });
    });

    it('POST /spellbook/changePage', (done) => {
      chai.request(app)
      .post('/spellbook/changePage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send(page)
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        chai.assert.equal(res.body.current.id, page.id);
        chai.assert.lengthOf(res.body.recommendation, 1);
        done();
      });
    });
  });

  describe('Generate some other arcanes', () => {
    it('POST /users/authenticate', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isDefined(res.body.token);
        chai.assert.equal(res.body.user.username, 'user');
        chai.assert.equal(res.body.user.email, 'user@mail.com');
        chai.assert.isUndefined(res.body.error);
        authToken = res.body.token;
        done();
      });
    });

    it('POST /generate', (done) => {
      formula.size = [{type: "C", value: 12}];
      chai.request(app)
      .post('/generate')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        content: formula
      })
      .end((err, res) => {
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);

        formula.size = [{type: "C", value: 16}];
        chai.request(app)
        .post('/generate')
        .set('Authorization', authToken)
        .set('Content-Type', 'application/json')
        .set('Cache-control', 'no-cache')
        .send({
          content: formula
        })
        .end((err, res) => {
          chai.assert.isNull(err, err);
          chai.assert.isTrue(res.body.success);

          formula.size = [{type: "C", value: 20}];
          chai.request(app)
          .post('/generate')
          .set('Authorization', authToken)
          .set('Content-Type', 'application/json')
          .set('Cache-control', 'no-cache')
          .send({
            content: formula
          })
          .end((err, res) => {
            chai.assert.isNull(err, err);
            chai.assert.isTrue(res.body.success);
            done();
          });
        });
      });
    }).timeout(5000);
  });

  describe('Second user should have more recommendations', () => {
    it('POST /users/authenticate', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user2',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isDefined(res.body.token);
        chai.assert.equal(res.body.user.username, 'user2');
        chai.assert.equal(res.body.user.email, 'user2@mail.com');
        authToken = res.body.token;
        done();
      });
    });

    it('POST /spellbook/changePage', (done) => {
      chai.request(app)
      .post('/spellbook/changePage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send(page)
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        chai.assert.equal(res.body.current.id, page.id);
        chai.assert.lengthOf(res.body.recommendation, 3);
        recom = res.body.recommendation[0];
        done();
      });
    });
  });

  describe('Add a recommendation', () => {
    it('POST /spellbook/newPage', (done) => {
      chai.request(app)
      .post('/spellbook/newPage')
      .set('Authorization', authToken)
      .set('content-type', 'text/plain')
      .set('Cache-control', 'no-cache')
      .send("c")
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        page = res.body;
        done();
      });
    });

    it('POST /spellbook/addToPage', (done) => {
      chai.request(app)
      .post('/spellbook/addToPage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        pages: [page],
        arcaneId: recom.id,
        pageId: page.id,
        recommendation: true
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.recommendation);

        chai.request(app)
        .post('/spellbook/changePage')
        .set('Authorization', authToken)
        .set('Content-Type', 'application/json')
        .set('Cache-control', 'no-cache')
        .send(page)
        .end((err, res) => {
          chai.expect(res).to.have.status(200);
          chai.assert.isNull(err, err);
          chai.assert.isDefined(res.body.current);
          chai.assert.equal(res.body.current.id, page.id);
          chai.assert.lengthOf(res.body.recommendation, 3);
          chai.assert.lengthOf(res.body.current.content, 1);
          recom = res.body.recommendation[0];
          done();
        });
      });
    });
  });

  describe('Add an other recommendation', () => {
    it('POST /spellbook/addToPage', (done) => {
      chai.request(app)
      .post('/spellbook/addToPage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        pages: [page],
        arcaneId: recom.id,
        pageId: page.id,
        recommendation: true
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.recommendation);

        chai.request(app)
        .post('/spellbook/changePage')
        .set('Authorization', authToken)
        .set('Content-Type', 'application/json')
        .set('Cache-control', 'no-cache')
        .send(page)
        .end((err, res) => {
          chai.expect(res).to.have.status(200);
          chai.assert.isNull(err, err);
          chai.assert.isDefined(res.body.current);
          chai.assert.equal(res.body.current.id, page.id);
          chai.assert.lengthOf(res.body.recommendation, 2);
          chai.assert.lengthOf(res.body.current.content, 2);
          done();
        });
      });
    });
  });

  describe('Refuse recommendation', () => {
    it('POST /spellbook/changePage', (done) => {
      chai.request(app)
      .post('/spellbook/changePage')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send(page)
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.current);
        chai.assert.equal(res.body.current.id, page.id);
        recom = res.body.recommendation[0];
        page = res.body.current;
        done();
      });
    });

    it('POST /spellbook/deleteFromPage', (done) => {
      chai.request(app)
      .post('/spellbook/deleteFromPage')
      .set('Authorization', authToken)
      .set('content-type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        pageId: page.id,
        arcaneId: recom.id,
        recommendation: true
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.lengthOf(res.body.recommendation, 1);
        chai.assert.notEqual(res.body.recommendation[0].id, recom.id);
        done();
      });
    });
  });

  describe('Delete an arcane from the page', () => {
    it('POST /spellbook/deleteFromPage', (done) => {
      chai.request(app)
      .post('/spellbook/deleteFromPage')
      .set('Authorization', authToken)
      .set('content-type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        pageId: page.id,
        arcaneId: page.content[0].id,
        recommendation: false
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        done();
      });
    });
  });

  describe('Delete a page containing arcanes and recommendations', () => {
    it('GET /spellbook', (done) => {
      chai.request(app)
      .get('/spellbook')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isDefined(res.body.list);
        list = res.body.list;
        done();
      });
    });

    it('POST /spellbook/deletePage', (done) => {
      chai.request(app)
      .post('/spellbook/deletePage')
      .set('Authorization', authToken)
      .set('content-type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        id: page.id,
        list: list
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isUndefined(res.body.recommendation);
        chai.assert.equal(res.body.page.name, "Arcanes connus");
        done();
      });
    });
  });

});
