const chai = require('chai');
const chaiHttp = require('chai-http');
const app = require('../../server').app;

chai.use(chaiHttp);

module.exports = describe('users routes', () => {
  let authToken = undefined;

  describe('POST register', () => {
    it('Should register a new user', (done) => {
      chai.request(app)
      .post('/users/register')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          email: 'user@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });
  });

  describe('POST authenticate', () => {
    it('Should not find a non-existing user', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'notAUser',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isFalse(res.body.success);
        chai.assert.isDefined(res.body.msg);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('Should not connect with the wrong password', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          password: 'WrongPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isFalse(res.body.success);
        chai.assert.isDefined(res.body.msg);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('Should connect with the right username and password', (done) => {
      chai.request(app)
      .post('/users/authenticate')
      .set('content-type', 'application/json')
      .send({
          username: 'user',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isDefined(res.body.token);
        chai.assert.equal(res.body.user.username, 'user');
        chai.assert.equal(res.body.user.email, 'user@mail.com');
        chai.assert.isUndefined(res.body.error);
        authToken = res.body.token;
        done();
      });
    });
  });

  describe('GET profile', () => {
    it('Should get user information', (done) => {
      chai.request(app)
      .get('/users/profile')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.equal(res.body.user.username, 'user');
        chai.assert.equal(res.body.user.email, 'user@mail.com');
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });
  });

  describe('POST updateUserInfo', () => {
    it('Should not update if passwords do not match', (done) => {
      chai.request(app)
      .post('/users/updateUserInfo')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
          type: 'email',
          email: 'user@mail.com',
          password: 'WrongPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isFalse(res.body.success);
        chai.assert.isDefined(res.body.msg);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('Should not register existing email', (done) => {
      chai.request(app)
      .post('/users/updateUserInfo')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
          type: 'email',
          email: 'user@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isFalse(res.body.success);
        chai.assert.isDefined(res.body.msg);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('Should register new email', (done) => {
      chai.request(app)
      .post('/users/updateUserInfo')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
          type: 'email',
          email: 'new@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });

    it('Should register new password', (done) => {
      chai.request(app)
      .post('/users/updateUserInfo')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
          type: 'password',
          newPassword: 'NewPwd123',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.assert.isTrue(res.body.success);
        chai.assert.isUndefined(res.body.error);
        done();
      });
    });
  });
});
